/*
 Copyright (c) 2011 Rane Corporation. All Rights Reserved.
 Abstract:
 This header provides prototypes and related constants and structures for all
 control functions available for the Rane Sixty-Two audio driver stack.
 */

#ifndef h_RH62Api
#define h_RH62Api

#ifdef _WIN32
// On Windows, use Microsoft C directive for importing from a DLL.
#ifndef RH62_API_EXTERN
#define RH62_API_EXTERN __declspec(dllimport)
#endif
#define RH62_API_CALLBACK_CONV __stdcall
#else
#define RH62_API_EXTERN extern
#define RH62_API_CALLBACK_CONV
#endif

#ifdef __cplusplus
extern "C" {
#endif
   
/**
 * Constants for Control DLL result error/success codes.
 * Zero is success, all nonzero are error or other non-completion conditions.
 */
typedef enum
{
   RH62_RESULT_SUCCESS               = 0,
   RH62_RESULT_UNSPECIFIED_ERROR     = 1,
   RH62_RESULT_UNIMPLEMENTED         = 2,
   RH62_RESULT_INVALID_HANDLE        = 3,
   RH62_RESULT_COMMUNICATION_ERROR   = 4,
   RH62_RESULT_INVALID_DRIVER_STATE  = 5,
   RH62_RESULT_DRIVER_ERROR          = 6,
   RH62_RESULT_ASYNC_CALL            = 7,
   RH62_RESULT_INVALID_PARAMETER     = 8,
   RH62_RESULT_INSUFFICIENT_SPACE    = 9,
   RH62_RESULT_EMPTY_PACKET          = 10,
   RH62_RESULT_TYPE_MISMATCH         = 11,
   RH62_RESULT_NO_DEVICE             = 12
} RH62_error_t;

/**
 * Constants for building MIDI strings.
 */
typedef enum
{
   RH62_MIDI_STATUS_UNKNOWN        = 0xFF,
   RH62_MIDI_STATUS_SYS_EX         = 0xF0,
   RH62_MIDI_STATUS_NOTE_ON        = 0x90,
   RH62_MIDI_STATUS_NOTE_OFF       = 0x80,
   RH62_MIDI_STATUS_CONTROL_CHANGE = 0xB0
} RH62_midi_status_command_t;

/** Rane-defined MIDI sysex command IDs */
typedef enum
{
   RH62_SYSEX_COMMAND_GET_ALL         = 0x04
} RH62_sysex_command_t;

/** Velocity constants for LEDs */
typedef enum
{
   RH62_H2D_VELOCITY_LED_OFF                = 0x00,
   RH62_H2D_VELOCITY_LED_PRIMARY_DIM        = 0x1F,
   RH62_H2D_VELOCITY_LED_PRIMARY_FULL_ON    = 0x30,
   RH62_H2D_VELOCITY_LED_SECONDARY_DIM      = 0x4F,
   RH62_H2D_VELOCITY_LED_SECONDARY_FULL_ON  = 0x60
} RH62_velocity_t;

/** Constants for Note On/Off Numbers */
typedef enum
{
   RH62_FX_ON_NOTE                     = 0x00,
   RH62_FX_ENCODER_NOTE                = 0x02,
   RH62_FX_CUE_NOTE                    = 0x04,
   RH62_FX_TAP_NOTE                    = 0x05,
   RH62_FX_JOYSTICK_NOTE               = 0x07,
   RH62_FX_REVERB_NOTE                 = 0x08,
   RH62_FX_ROBOT_NOTE                  = 0x09,
   RH62_FX_PHASER_NOTE                 = 0x0A,
   RH62_FX_FLANGER_NOTE                = 0x0B,
   RH62_FX_FILTER_NOTE                 = 0x0C,
   RH62_FX_ECHO_NOTE                   = 0x0D,
   RH62_LEFT_CUE_1_NOTE                = 0x12,
   RH62_LEFT_DELETE_NOTE               = 0x13,
   RH62_LEFT_CUE_5_NOTE                = 0x14,
   RH62_LEFT_CUE_4_NOTE                = 0x15,
   RH62_LEFT_CUE_3_NOTE                = 0x16,
   RH62_LEFT_CUE_2_NOTE                = 0x17,
   RH62_USB_AUX_FLEXFX_NOTE            = 0x18,
   RH62_LEFT_ENCODER_MANUAL_NOTE       = 0x19,
   RH62_LEFT_MANUAL_IN_NOTE            = 0x1C,
   RH62_LEFT_MANUAL_LOOP_NOTE          = 0x1D,
   RH62_LEFT_MANUAL_OUT_NOTE           = 0x1E,
   RH62_USB_INSERT_NOTE                = 0x1F,
   RH62_MIC_FLEXFX_NOTE                = 0x22,
   RH62_MIC_ON_NOTE                    = 0x29,
   RH62_LEFT_SP6_ASSIGN_NOTE           = 0x2A,
   RH62_LEFT_BACK_NOTE                 = 0x2B,
   RH62_LEFT_ENCODER_LOAD_NOTE         = 0x2C,
   RH62_LEFT_LOOPS_NOTE                = 0x2D,
   RH62_MIC_TALKOVER_NOTE              = 0x2E,
   RH62_RIGHT_LOOPS_NOTE               = 0x32,
   RH62_RIGHT_SP6_ASSIGN_NOTE          = 0x33,
   RH62_RIGHT_BACK_NOTE                = 0x34,
   RH62_RIGHT_ENCODER_LOAD_NOTE        = 0x37,
   RH62_SPLIT_CUE_NOTE                 = 0x38,
   RH62_RIGHT_MANUAL_LOOP_NOTE         = 0x39,
   RH62_RIGHT_ENCODER_MANUAL_NOTE      = 0x3A,
   RH62_RIGHT_MANUAL_OUT_NOTE          = 0x3D,
   RH62_RIGHT_MANUAL_IN_NOTE           = 0x3F,
   RH62_PGM2_FLEXFX_NOTE               = 0x43,
   RH62_PGM1_FLEXFX_NOTE               = 0x44,
   RH62_PGM1_CUE_NOTE                  = 0x45,
   RH62_PGM2_CUE_NOTE                  = 0x46,
   RH62_RIGHT_CUE_2_NOTE               = 0x49,
   RH62_RIGHT_CUE_3_NOTE               = 0x4A,
   RH62_RIGHT_CUE_4_NOTE               = 0x4B,
   RH62_RIGHT_CUE_5_NOTE               = 0x4C,
   RH62_USB_AUX_CUE_NOTE               = 0x4D,
   RH62_RIGHT_CUE_1_NOTE               = 0x4E,
   RH62_RIGHT_DELETE_NOTE              = 0x4F,
   RH62_CUES_SAMPLES_NOTE              = 0x5A,
   RH62_EXT_INSERT_NOTE                = 0x5B,
   RH62_SAMPLE_1_NOTE                  = 0x62,
   RH62_SAMPLE_2_NOTE                  = 0x63,
   RH62_SAMPLE_3_NOTE                  = 0x64,
   RH62_SAMPLE_4_NOTE                  = 0x65,
   RH62_SAMPLE_5_NOTE                  = 0x66,
   RH62_SAMPLE_6_NOTE                  = 0x67,
   RH62_BANK_A_NOTE                    = 0x68,
   RH62_BANK_B_NOTE                    = 0x69,
   RH62_BANK_C_NOTE                    = 0x6A,
   RH62_BANK_D_NOTE                    = 0x6B,
   RH62_LEFT_ENCODER_AUTO_NOTE         = 0x6C,
   RH62_LEFT_AUTO_LOOP_NOTE            = 0x6D,
   RH62_LEFT_AUTO_ROLL_NOTE            = 0x6E,
   RH62_LEFT_AUTO_SAVE_NOTE            = 0x6F,
   RH62_RIGHT_ENCODER_AUTO_NOTE        = 0x70,
   RH62_RIGHT_AUTO_LOOP_NOTE           = 0x71,
   RH62_RIGHT_AUTO_ROLL_NOTE           = 0x72,
   RH62_RIGHT_AUTO_SAVE_NOTE           = 0x73
} RH62_onoff_t;

/** Constants for Control Change Numbers */
typedef enum
{
   RH62_CROSS_FADER                              = 0x00,
   RH62_FX_DEPTH                                 = 0x01,
   RH62_PGM1_FADER_CONTOUR                       = 0x02,
   RH62_PGM2_FADER_CONTOUR                       = 0x03,
   RH62_PGM1_PAN                                 = 0x04,
   RH62_PGM2_PAN                                 = 0x05,
   RH62_USB_AUX_LEVEL                            = 0x07,
   RH62_USB_AUX_FILTER                           = 0x08,
   RH62_LEFT_ENCODER_LOAD                        = 0x10,
   RH62_LEFT_ENCODER_MANUAL                      = 0x11,
   RH62_RIGHT_ENCODER_LOAD                       = 0x12,
   RH62_RIGHT_ENCODER_MANUAL                     = 0x13,
   RH62_FX_ENCODER                               = 0x14,
   RH62_PARAM_A1_PHONO_SENSITIVITY               = 0x15,
   RH62_PARAM_PGM1_FILTER_RESONANCE              = 0x16,
   RH62_PARAM_PGM1_RECORD_SELECT                 = 0x17,
   RH62_PARAM_A2_PHONO_SENSITIVITY               = 0x18,
   RH62_PARAM_PGM2_FILTER_RESONANCE              = 0x19,
   RH62_PARAM_PGM2_RECORD_SELECT                 = 0x1A,
   RH62_PARAM_A3_PHONO_SENSITIVITY               = 0x1B,
   RH62_PARAM_A4_PHONO_SENSITIVITY               = 0x1E,
   RH62_PARAM_MAIN_RECORD_SELECT                 = 0x21,
   RH62_PARAM_USB_AUX_FILTER_RESONANCE           = 0x26,
   RH62_PARAM_FX_FLANGER_TYPE                    = 0x27,
   RH62_PARAM_FX_ECHO_TYPE                       = 0x28,
   RH62_PARAM_FX_FILTER_TYPE                     = 0x29,
   RH62_PARAM_FX_LC_ECHO_FREQ                    = 0x2A,
   RH62_PARAM_FX_LC_HOLD_ECHO_FREQ               = 0x2B,
   RH62_LEFT_ENCODER_AUTO                        = 0x2C,
   RH62_RIGHT_ENCODER_AUTO                       = 0x2D,
   RH62_PARAM_USER_BUTTON_LIGHTING               = 0x2E,
   RH62_PARAM_ANLG_INSERT_SENSITIVITY            = 0x2F,
   RH62_PARAM_MIC_CLEAN_FEED                     = 0x30, // new
   RH62_PARAM_HEADPHONE_TONE                     = 0x31, // new
   RH62_PARAM_FX_DUCK_ECHO_FREQ                  = 0x37, // new 
   RH62_PGM1_FILTER                              = 0x41,
   RH62_PGM2_FILTER                              = 0x42,
   RH62_SESSION_IN_LEVEL                         = 0x45,
   RH62_PGM1_LOW_EQ                              = 0x47,
   RH62_PGM2_LOW_EQ                              = 0x48,
   RH62_BOOTH_OUT_LEVEL                          = 0x4B,
   RH62_PGM1_MID_EQ                              = 0x4D,
   RH62_PGM2_MID_EQ                              = 0x4E,
   RH62_MIC_LOW_EQ                               = 0x52,
   RH62_PGM1_FADER                               = 0x53,
   RH62_PGM2_FADER                               = 0x54,
   RH62_SESSION_OUT_LEVEL                        = 0x57,
   RH62_PGM1_LEVEL                               = 0x59,
   RH62_PGM2_LEVEL                               = 0x5A,
   RH62_MAIN_OUT_LEVEL                           = 0x5D,
   RH62_MIC_HIGH_EQ                              = 0x5E,
   RH62_CROSS_FADER_CONTOUR                      = 0x62,
   RH62_PHONES_LEVEL                             = 0x63,
   RH62_MIC_LEVEL                                = 0x64,
   RH62_PGM1_HIGH_EQ                             = 0x65,
   RH62_PGM2_HIGH_EQ                             = 0x66,
   RH62_PHONES_PAN                               = 0x69,
   RH62_CROSS_FADER_REVERSE                      = 0x6A,
   RH62_PGM1_FADER_REVERSE                       = 0x6B,
   RH62_PGM2_FADER_REVERSE                       = 0x6C,
   RH62_FX_JOYSTICK                              = 0x6F,
   RH62_INPUT_A1_TYPE                            = 0x70,
   RH62_INPUT_A2_TYPE                            = 0x71,
   RH62_INPUT_A3_TYPE                            = 0x72,
   RH62_INPUT_A4_TYPE                            = 0x73,
   RH62_PGM1_SOURCE                              = 0x74,
   RH62_PGM2_SOURCE                              = 0x75
} RH62_controlchange_t;

/** MIDI Channel Type */
typedef enum 
{
   RH62_MIDI_OFF    = 0, 
   RH62_MIDI_CH_1   = 1, 
   RH62_MIDI_CH_2   = 2, 
   RH62_MIDI_CH_3   = 3, 
   RH62_MIDI_CH_4   = 4, 
   RH62_MIDI_CH_5   = 5, 
   RH62_MIDI_CH_6   = 6, 
   RH62_MIDI_CH_7   = 7, 
   RH62_MIDI_CH_8   = 8, 
   RH62_MIDI_CH_9   = 9, 
   RH62_MIDI_CH_10  = 10, 
   RH62_MIDI_CH_11  = 11, 
   RH62_MIDI_CH_12  = 12, 
   RH62_MIDI_CH_13  = 13,
   RH62_MIDI_CH_14  = 14, 
   RH62_MIDI_CH_15  = 15, 
   RH62_MIDI_CH_16  = 16
} RH62_midi_ch_t;

/** Length in bytes of bridge message data */
const int RH62_BRIDGE_MESSAGE_LENGTH     = 59;

/** Maximum length in bytes of HIDI message data */
const int RH62_MAX_HIDI_MESSAGE_LENGTH   = 58;

/** Number of bytes in timer challenge/response block. */
const int RH62_DRIVER_TIMER_LENGTH       = 8;

/** Rane ID used for MIDI sysex sequences */
const int RH62_MIDI_SYSEX_RANE_ID        = 0x17;

/** EOX byte used for MIDI sysex sequences */
const int RH62_MIDI_SYSEX_EOX            = 0xF7;

/**
 * Structure returned for all version info queries.
 * Note: any unused fields are set to zero.
 */
typedef struct
{
   int major;
   int minor;
   int sub;
   int build;
} RH62_version_t;

/** USB Port Type */
typedef enum
{
   RH62_USB_PORT_NONE = 0,
   RH62_USB_PORT_A    = 1,
   RH62_USB_PORT_B    = 2
} RH62_usb_port_t;

/** USB Port Speed Type */
typedef enum
{
   RH62_USB_FULL_SPEED = 1,
   RH62_USB_HIGH_SPEED = 2
} RH62_usb_port_speed_t;

/** Data structure returned for connection status callbacks and port status function */
typedef struct
{
   RH62_usb_port_t		thisPort;
   RH62_usb_port_t		otherPort;
} RH62_usb_port_status_t;

/** Data structure returned for update firmware callbacks */
typedef struct
{
   unsigned char progress; /* Current progress indicator for firmware updates,
                            int range 0 to 100 = percent complete */
} RH62_firmware_status_data_t;

/** Data structure returned for HIDI msg callbacks */
typedef struct
{
   unsigned char length;
   unsigned char msg[RH62_MAX_HIDI_MESSAGE_LENGTH];
} RH62_hidimsg_data_t;

/** Data structure returned for bridge packet callbacks */
typedef struct
{
   unsigned char msg[RH62_BRIDGE_MESSAGE_LENGTH];
} RH62_bridgepacket_data_t;

/** Data structure used for MIDI Channel functions */ 
typedef struct 
{
   RH62_midi_ch_t	inChannel;
   RH62_midi_ch_t	outChannel; 
} RH62_midi_ch_data_t;

/** Data structure used for MIDI Beat Clock functions */ 
typedef struct 
{
   int	sendClock;
   int	receiveClock; 
} RH62_midi_beat_clock_data_t;

/** Application-defined HIDI message callback function */
typedef void (RH62_API_CALLBACK_CONV RH62_hidimsg_callback_t) (RH62_hidimsg_data_t* data);
/** Application-defined USB port status callback function */
typedef void (RH62_API_CALLBACK_CONV RH62_usb_port_status_callback_t) (RH62_usb_port_status_t* data);
/** Application-defined firmware progress callback function */
typedef void (RH62_API_CALLBACK_CONV RH62_firmware_callback_t) (RH62_firmware_status_data_t* data);
/** Application-defined bridge packet receipt callback function */
typedef void (RH62_API_CALLBACK_CONV RH62_bridgepacket_callback_t) (RH62_bridgepacket_data_t* data);
/** Application-defined MIDI channel change callback function */
typedef void (RH62_API_CALLBACK_CONV RH62_midi_ch_callback_t) (RH62_midi_ch_data_t* data);
/** Application-defined MIDI Beat Clock change callback function */
typedef void (RH62_API_CALLBACK_CONV RH62_midi_beat_clock_callback_t) (RH62_midi_beat_clock_data_t* data);


/**
 * Returns a new handle, even if the device is not present.
 * Only returns NULL on catastrophic failure.
 */
RH62_API_EXTERN void* RH62_open();

/** Closes an open device. */
RH62_API_EXTERN void RH62_close(void* handle);

/** Causes all pending HIDI messages to be sent to the device before 
 returning from this call. */
RH62_API_EXTERN int RH62_flush_hidi_queue(void* handle);

/** Determines whether the device is a boot device. */
RH62_API_EXTERN int RH62_is_boot_device(
                                        void*   handle,
                                        int*    is_boot);

/** Retrieves the USB port speed of the device. */
RH62_API_EXTERN int RH62_get_port_speed(
                                        void*                 handle,
                                        RH62_usb_port_speed_t*  speed);

/** Retrieves USB port connection information */
RH62_API_EXTERN int RH62_get_usb_port_status(
                                             void*                   handle,
                                             RH62_usb_port_status_t*   data);

/** Sets a new callback function for the connection status event. */
RH62_API_EXTERN int RH62_set_usb_port_status_callback(
                                                      void*							      handle,
                                                      RH62_usb_port_status_callback_t*  callback);

/** Retrieves the Vendor ID of the device. */
RH62_API_EXTERN int RH62_get_vendor_id(
                                       void*   handle,
                                       int*    value);

/** Retrieves the Product ID of the device. */
RH62_API_EXTERN int RH62_get_product_id(
                                        void*   handle,
                                        int*    value);

/** Retrieves API version information. */
RH62_API_EXTERN int RH62_get_api_version(RH62_version_t* value);

/** Retrieves the driver version string. */
RH62_API_EXTERN int RH62_get_driver_version(
                                            void*         handle,
                                            RH62_version_t* value);

/** Performs some validation. */
RH62_API_EXTERN int RH62_driver_timer(
                                      void*                  handle,
                                      const unsigned char*   data,
                                      int                    size,
                                      unsigned char*         response,
                                      int                    response_size);

/** Retrieves the firmware version information. */
RH62_API_EXTERN int RH62_get_firmware_version(
                                              void*         handle,
                                              RH62_version_t* value);

/** Retrieves the firmware version embedded in the control API library */
RH62_API_EXTERN int RH62_get_firmware_embedded_version(
                                                       void*         handle,
                                                       RH62_version_t* version);

/**
 * Sends a message to trigger the device to send all control and function
 *  parameters. Control and function parameters will be sent by the device as
 *  HIDI messages.
 */
RH62_API_EXTERN int RH62_get_all_controls(void* handle);

/**
 * Enqueues a HIDI packet to send to the device
 */
RH62_API_EXTERN int RH62_send_hidimsg(void* handle, RH62_hidimsg_data_t* data);

/** Adds a MIDI Note On message to the outgoing queue. */
RH62_API_EXTERN int RH62_send_note_on(
                                      void*         handle,
                                      unsigned char note,
                                      unsigned char velocity);

/** Adds a MIDI Note Off message to the outgoing queue. */
RH62_API_EXTERN int RH62_send_note_off(
                                       void*         handle,
                                       unsigned char note,
                                       unsigned char velocity);

/** Adds a MIDI Control Change message to the outgoing queue. */
RH62_API_EXTERN int RH62_send_control_change(
                                             void*         handle,
                                             unsigned char control,
                                             unsigned char value);

/**
 * Adds a MIDI System Exclusive message to the outgoing queue.  Note that
 * the data and length parameters should reflect only the payload of the SYS_EX
 * sequence.  Thus for example, to send the sequence...
 *    0xF0, 0x17, 0x52, 0x00, 0xF7
 * ... the caller must supply only (0x52, 0x00) in "data", with "length" == 2,
 * and the library will automatically add the SYS_EX status byte, Rane ID,
 * and EOX).
 */
RH62_API_EXTERN int RH62_send_sys_ex(
                                     void*            handle,
                                     unsigned char*   data,
                                     unsigned char    length);

/** Sets a new callback function for the hidi msg event. */
RH62_API_EXTERN int RH62_set_hidimsg_callback(
                                              void*                  handle,
                                              RH62_hidimsg_callback_t* callback);

/** Sends a bridge packet. */
RH62_API_EXTERN int RH62_send_bridge_packet(
                                            void*                     handle,
                                            RH62_bridgepacket_data_t*   data);

/** Sets a new callback function for the bridge packet event. */
RH62_API_EXTERN int RH62_set_bridgepacket_callback(
                                                   void*                        handle,
                                                   RH62_bridgepacket_callback_t*  callback);

/** Updates the device firmware from a memory buffer. */
RH62_API_EXTERN int RH62_set_update_firmware(
                                             void*                     handle,
                                             const unsigned char*      data,
                                             int                       size,
                                             RH62_firmware_callback_t*   callback);

/** Updates the device firmware from the copy embedded in the DLL. */
RH62_API_EXTERN int RH62_set_update_firmware_from_embedded(
                                                           void*                     handle,
                                                           RH62_firmware_callback_t*   callback);

/**
 * Retrieves the audio buffer size, a component of latency.
 * Note that this only includes software latency, and excludes
 * latency from hardware.
 */
RH62_API_EXTERN int RH62_get_buffer_millisecs(
                                              void*   handle,
                                              int*    millisecs);

/**
 * Modifies the audio buffer size, directly affecting latency
 * Note that this only includes software latency, and excludes
 * latency from hardware.
 */
RH62_API_EXTERN int RH62_set_buffer_millisecs(
                                              void*   handle,
                                              int     millisecs);

/**
 * Instructs device to disconnect and reconnect with alternate PID.
 */
RH62_API_EXTERN int RH62_set_alt_product_id(void* handle);

/** Gets the current state of the MIDI channels */ 
RH62_API_EXTERN int RH62_get_midi_channels(void* handle, RH62_midi_ch_data_t* midiChannels);

/** Sets the current state of the MIDI channels */ 
RH62_API_EXTERN int RH62_set_midi_channels(void* handle, RH62_midi_ch_data_t* midiChannels);

/** Sets a new callback function for the MIDI channel change event. */ 
RH62_API_EXTERN int RH62_set_midi_ch_callback(void* handle, RH62_midi_ch_callback_t* callback);

/** Gets the current state of the MIDI Beat Clock **/
RH62_API_EXTERN int RH62_get_midi_beat_clock(void* handle, RH62_midi_beat_clock_data_t* data);

/** Sets the current state of the MIDI Beat Clock**/
RH62_API_EXTERN int RH62_set_midi_beat_clock(void* handle, RH62_midi_beat_clock_data_t* data);

/** Sets a new callback function for the MIDI Beat Clock change event. */ 
RH62_API_EXTERN int RH62_set_midi_beat_clock_callback(void* handle, RH62_midi_beat_clock_callback_t* callback);
   
#ifdef __cplusplus
} // (end extern "C")
#endif

#endif
